@@ -185,10 +185,12 @@ class Conv(keras.Model):
     # PUBLIC
     #
     def __init__(self,
-            filters,
+            filters=None,
             kernel_size=3,
             padding='same',
             seperable=False,
+            pointwise_first=False,
+            depthwise=False,
             norm=BATCH_NORM,
             norm_config={},
             act=True,
@@ -207,12 +209,6 @@ class Conv(keras.Model):
         self.paddings=False
         self.block_name=name or self.name
         self.named_layers=named_layers
-        if seperable:
-            _conv=layers.SeparableConv2D
-            cname='sepconv2d'
-        else:
-            _conv=layers.Conv2D
-            cname='conv2d'
         if dilation_rate>1:
             strides=1
         if kernel_size in [1,(1,1)]:
@@ -228,8 +224,10 @@ class Conv(keras.Model):
                 [p,p],
                 [0,0]])
             padding='valid'
+        _conv, cname, filters=self._conv_setup(depthwise,seperable,pointwise_first,filters)
+        if filters:
+            conv_config['filters']=filters
         self.conv=_conv(
-            filters=filters,
             kernel_size=kernel_size,
             padding=padding,
             dilation_rate=dilation_rate,
@@ -248,6 +246,8 @@ class Conv(keras.Model):
         self.norm=self._norm(norm,norm_config)
         self.act=self._activation(act,act_config)
         self.do=self._dropout(dropout,dropout_config)
+
+
     def __call__(self,x):
         for m in self.order:
             l=getattr(self,m)
@@ -261,6 +261,26 @@ class Conv(keras.Model):
     #
     # INTERNAL
     #
+    def _conv_setup(self,depthwise,seperable,pointwise_first,filters):
+        if depthwise:
+            _conv=layers.DepthwiseConv2D
+            cname='depthconv2d'
+            filters=None
+        else:
+            if seperable:
+                if pointwise_first:
+                    _conv=XSeparableConv2D
+                    cname='xsepconv2d'
+                else:
+                    _conv=layers.SeparableConv2D
+                    cname='sepconv2d'
+            else:
+                _conv=layers.Conv2D
+                cname='conv2d'
+        return _conv, cname, filters
+
+
+
     def _layer_name(self,name):
         return layer_name(self.block_name,name,named=self.named_layers)
 
@@ -298,6 +318,107 @@ class Conv(keras.Model):
 
 
 
+class XSeparableConv2D(keras.Model):
+    """ Xception Style Seperable Conv (pointwise first)
+    """
+    #
+    # PUBLIC
+    #
+    def __init__(self,filters,**conv_config):
+        super(XSeparableConv2D, self).__init__()
+        self.pointwise=layers.Conv2D(filters=filters,kernel_size=1)
+        self.depthwise=layers.DepthwiseConv2D(**conv_config)
+
+    def __call__(self,x):
+        x=self.pointwise(x)
+        return self.depthwise(x)
+
+
+class BandMath(keras.Model):
+    """ BandMath: Let the model learn band indices
+    """
+    DEFAULT_ORDER=['act','norm']
+    BATCH_NORM='batch'
+    GROUP_NORM='group'
+    DEFAULT_NORM=BATCH_NORM
+    #
+    # PUBLIC
+    #
+    def __init__(self,filters,
+            depth=1,
+            norm=DEFAULT_NORM,
+            norm_config={},
+            act=True,
+            act_config={},
+            conv_act=False,
+            conv_norm=False,
+            name=None,
+            order=DEFAULT_ORDER,
+            **conv_config):
+        super(BandMath, self).__init__()
+        self.block_name=name or self.name
+        self.numerator_stack=[
+            Conv(
+                filters=filters,
+                kernel_size=1,
+                act=conv_act,
+                norm=conv_norm,
+                name=layer_name(f'{self.block_name}_n',index=i),
+                **conv_config)
+            for i in range(depth)]
+        self.denominator_stack=[
+            Conv(
+                filters=filters,
+                kernel_size=1,
+                act=conv_act,
+                norm=conv_norm,
+                name=layer_name(f'{self.block_name}_d',index=i),
+                **conv_config)
+            for i in range(depth)]
+        self.order=order
+        self.norm=self._norm(norm,norm_config)
+        self.act=self._activation(act,act_config)
+
+
+    def __call__(self,x):
+        numerator=x
+        for l in self.numerator_stack:
+            numerator=l(numerator)
+        denominator=x
+        for l in self.denominator_stack:
+            denominator=l(denominator)
+        x=numerator/(denominator+EPS)
+        for m in self.order:
+            l=getattr(self,m)
+            if l: x=l(x)
+        return x
+
+
+    def _norm(self,norm,norm_config):
+        if norm==True:
+            norm=Conv.DEFAULT_NORM
+        if norm==Conv.BATCH_NORM:
+            norm=layers.BatchNormalization(
+                name=layer_name(f'{self.block_name}_bn'),
+                **norm_config)
+        elif norm==Conv.GROUP_NORM:
+            norm=addons.GroupNormalization(
+                name=layer_name(f'{self.block_name}_gn'),
+                **norm_config)
+        elif norm:
+            raise ValueError('only `group` and `batch` norms implemented')
+        else:
+            norm=False
+        return norm      
+
+
+    def _activation(self,act,config):
+        if act:
+            act=get_activation(act,name=layer_name(f'{self.block_name}_act'),**config)
+        return act
+
+
+
 class Stack(keras.Model):
     """ (Res)Stack of Conv Blocks
     """
@@ -331,6 +452,8 @@ class Stack(keras.Model):
             residual_norm_config=None,
             seperable_residual=False,
             seperable=False,
+            pointwise_first=False,
+            depthwise_out=False,
             norm=True,
             norm_config={},
             output_dropout=True,
@@ -372,6 +495,8 @@ class Stack(keras.Model):
             max_pooling,
             seperable=seperable,
             seperable_residual=seperable_residual,
+            depthwise_out=depthwise_out,
+            pointwise_first=pointwise_first,
             norm=norm,
             norm_config=norm_config,
             padding=padding,
@@ -460,6 +585,7 @@ class Stack(keras.Model):
             max_pooling,
             seperable,
             seperable_residual,
+            depthwise_out,
             norm,
             norm_config,
             **shared_config):
@@ -471,6 +597,7 @@ class Stack(keras.Model):
             max_pooling)
         self.seperable=seperable
         self.seperable_residual=seperable_residual
+        self.depthwise_out=depthwise_out
         self.norm=norm
         self.norm_config=norm_config
         shared_config['dilation_rate']=dilation_rate
@@ -535,6 +662,11 @@ class Stack(keras.Model):
             dilation_rate=cfig.pop('dilation_rate',1)
             if (i==0):
                 dilation_rate=input_dilation_rate or dilation_rate
+                seperable=self.seperable
+                depthwise=False
+            elif self.depthwise_out:
+                seperable=False
+                depthwise=True
             if (i==last_layer_index):
                 if not self.output_dropout:
                     cfig['dropout']=False
@@ -546,7 +678,8 @@ class Stack(keras.Model):
                 kernel_size=k,
                 strides=strides,
                 dilation_rate=dilation_rate,
-                seperable=self.seperable,
+                seperable=seperable,
+                depthwise=depthwise,
                 act=self.act,
                 norm=self.norm,
                 norm_config=self.norm_config,
@@ -774,16 +907,25 @@ class BlockSqueezeExcitation(keras.Model):
     """ BlockSqueezeExcitation
     """
 
-    def __init__(self,filters,ratio=16,pool_type='avg',size=None,grid_size=3):
+    def __init__(self,
+            filters,
+            ratio=16,
+            pool_type='avg',
+            pool_size=None,
+            size=None,
+            grid_size=3):
         super(BlockSqueezeExcitation, self).__init__()
         if pool_type in ['avg','max']:
             self.pool_type=pool_type
         else:
             raise ValueError('BlockSqueezeExcitation: pooling must be max or avg')
-        self.grid_size=grid_size
-        self._set_pool_up(size)     
+        self.pooling=None
         self.reduce=layers.Dense(filters//ratio,activation='relu')
         self.expand=layers.Dense(filters,activation='sigmoid')
+        self.up=None
+        self.grid_size=grid_size
+        self.pool_size=pool_size
+        self._set_pool_up(size)
 
 
     def _get_pool_layer(self):
@@ -794,25 +936,26 @@ class BlockSqueezeExcitation(keras.Model):
  
 
     def _set_pool_up(self,size):
-        if size:
+        if self.pool_size:
+            self.pooling=self._get_pool_layer()(self.pool_size,padding='same',strides=1)
+            self.up=False
+        elif size:
             _ps=size/self.grid_size
             self.pool_size=int(_ps)
-            if _ps==self.pool_size:
-                self.pooling=self._get_pool_layer()(self.pool_size,padding='same')
-                self.up=layers.UpSampling2D(self.pool_size)
-            else:
-                raise ValueError('BlockSqueezeExcitation: grid_size/size must an integer')     
-        else:
-            self.pool_size=None
+            if _ps!=self.pool_size:
+                raise ValueError('BlockSqueezeExcitation: grid_size/size must an integer') 
+            self.pooling=self._get_pool_layer()(self.pool_size,padding='same')
+            self.up=layers.UpSampling2D(self.pool_size)
 
 
     def __call__(self,x,training=False):
-        if not self.pool_size:
+        if not self.pooling:
             self._set_pool_up(x.shape[1])  
         y=self.pooling(x)
         y=self.reduce(y)
         y=self.expand(y)
-        y=self.up(y)
+        if self.up:
+            y=self.up(y)
         return layers.multiply([x,y])
 
 
@@ -1080,6 +1223,7 @@ BLOCKS={
     'residual': Residual,
     'squeeze_excitation': SqueezeExcitation,
     'aspp': ASPP,
+    'band_math': BandMath,
     'segment_classifier': SegmentClassifier
 }
 ACTIVATIONS={
